@@ -1,8 +1,8 @@
 /****************************************************************************
 
- Copyright (C) 2002-2008 Gilles Debunne. All rights reserved.
+ Copyright (C) 2002-2011 Gilles Debunne. All rights reserved.
 
- This file is part of the QGLViewer library version 2.3.4.
+ This file is part of the QGLViewer library version 2.3.17.
 
  http://www.libqglviewer.com - contact@libqglviewer.com
 
@@ -96,9 +96,9 @@ public:
 			// MOC_SKIP_END
 # endif
 			defaultConstructor(); }
-#endif
 
-#if QT_VERSION >= 0x040000
+#else
+
 	explicit QGLViewer(QWidget* parent=0, const QGLWidget* shareWidget=0, Qt::WFlags flags=0);
 	explicit QGLViewer(QGLContext *context, QWidget* parent=0, const QGLWidget* shareWidget=0, Qt::WFlags flags=0);
 	explicit QGLViewer(const QGLFormat& format, QWidget* parent=0, const QGLWidget* shareWidget=0, Qt::WFlags flags=0);
@@ -141,15 +141,15 @@ public:
 	bool cameraIsEdited() const { return cameraIsEdited_; }
 
 
-	public slots:
+	public Q_SLOTS:
 		/*! Sets the state of axisIsDrawn(). Emits the axisIsDrawnChanged() signal. See also toggleAxisIsDrawn(). */
-		void setAxisIsDrawn(bool draw=true) { axisIsDrawn_ = draw; emit axisIsDrawnChanged(draw); if (updateGLOK_) updateGL(); };
+		void setAxisIsDrawn(bool draw=true) { axisIsDrawn_ = draw; Q_EMIT axisIsDrawnChanged(draw); if (updateGLOK_) updateGL(); };
 		/*! Sets the state of gridIsDrawn(). Emits the gridIsDrawnChanged() signal. See also toggleGridIsDrawn(). */
-		void setGridIsDrawn(bool draw=true) { gridIsDrawn_ = draw; emit gridIsDrawnChanged(draw); if (updateGLOK_) updateGL(); };
+		void setGridIsDrawn(bool draw=true) { gridIsDrawn_ = draw; Q_EMIT gridIsDrawnChanged(draw); if (updateGLOK_) updateGL(); };
 		/*! Sets the state of FPSIsDisplayed(). Emits the FPSIsDisplayedChanged() signal. See also toggleFPSIsDisplayed(). */
-		void setFPSIsDisplayed(bool display=true) { FPSIsDisplayed_ = display; emit FPSIsDisplayedChanged(display); if (updateGLOK_) updateGL(); };
+		void setFPSIsDisplayed(bool display=true) { FPSIsDisplayed_ = display; Q_EMIT FPSIsDisplayedChanged(display); if (updateGLOK_) updateGL(); };
 		/*! Sets the state of textIsEnabled(). Emits the textIsEnabledChanged() signal. See also toggleTextIsEnabled(). */
-		void setTextIsEnabled(bool enable=true) { textIsEnabled_ = enable; emit textIsEnabledChanged(enable); if (updateGLOK_) updateGL(); };
+		void setTextIsEnabled(bool enable=true) { textIsEnabled_ = enable; Q_EMIT textIsEnabledChanged(enable); if (updateGLOK_) updateGL(); };
 		void setCameraIsEdited(bool edit=true);
 
 		/*! Toggles the state of axisIsDrawn(). See also setAxisIsDrawn(). */
@@ -198,7 +198,7 @@ public:
 
 	See also backgroundColor(). */
 	QColor foregroundColor() const { return foregroundColor_; };
-	public slots:
+	public Q_SLOTS:
 		/*! Sets the backgroundColor() of the viewer and calls \c qglClearColor(). See also
 		setForegroundColor(). */
 		void setBackgroundColor(const QColor& color) { backgroundColor_=color; qglClearColor(color); };
@@ -232,7 +232,7 @@ public:
 	Do not mismatch this value (that only depends on the scene) with the qglviewer::Camera::revolveAroundPoint(). */
 	qglviewer::Vec sceneCenter() const { return camera()->sceneCenter(); }
 
-	public slots:
+	public Q_SLOTS:
 		/*! Sets the sceneRadius().
 
 		The camera() qglviewer::Camera::flySpeed() is set to 1% of this value by this method. Simple
@@ -249,8 +249,8 @@ public:
 
 		This is equivalent to:
 		\code
-		setSceneCenter((m+M)/2.0);
-		setSceneRadius(0.5*(M-m).norm());
+		setSceneCenter((min+max) / 2.0);
+		setSceneRadius((max-min).norm() / 2.0);
 		\endcode */
 		void setSceneBoundingBox(const qglviewer::Vec& min, const qglviewer::Vec& max) { camera()->setSceneBoundingBox(min,max); }
 
@@ -279,7 +279,7 @@ public:
 	Default value is \c NULL, meaning that no qglviewer::ManipulatedFrame is set. */
 	qglviewer::ManipulatedFrame* manipulatedFrame() const { return manipulatedFrame_; };
 
-	public slots:
+	public Q_SLOTS:
 		void setCamera(qglviewer::Camera* const camera);
 		void setManipulatedFrame(qglviewer::ManipulatedFrame* frame);
 		//@}
@@ -317,7 +317,7 @@ public:
 	You can also use qglviewer::MouseGrabber::removeFromMouseGrabberPool() to completely disable a
 	MouseGrabber in all the QGLViewers. */
 	bool mouseGrabberIsEnabled(const qglviewer::MouseGrabber* const mouseGrabber) { return !disabledMouseGrabbers_.contains(reinterpret_cast<size_t>(mouseGrabber)); };
-	public slots:
+	public Q_SLOTS:
 		void setMouseGrabber(qglviewer::MouseGrabber* mouseGrabber);
 		//@}
 
@@ -362,14 +362,13 @@ public:
 	See Camera::loadProjectionMatrixStereo() and Camera::loadModelViewMatrixStereo().
 
 	The stereo parameters are defined by the camera(). See qglviewer::Camera::setIODistance(),
-	qglviewer::Camera::setPhysicalDistanceToScreen(),
 	qglviewer::Camera::setPhysicalScreenWidth() and
 	qglviewer::Camera::setFocusDistance(). */
 	bool displaysInStereo() const { return stereo_; }
 	/*! Returns the recommended size for the QGLViewer. Default value is 600x400 pixels. */
 	virtual QSize sizeHint() const { return QSize(600, 400); }
 
-	public slots:
+	public Q_SLOTS:
 		void setFullScreen(bool fullScreen=true);
 		void setStereoDisplay(bool stereo=true);
 		/*! Toggles the state of isFullScreen(). See also setFullScreen(). */
@@ -443,7 +442,7 @@ public:
 	You need to setMouseTracking() to \c true in order to use MouseGrabber (see mouseGrabber()). See
 	details in the QWidget documentation. */
 	bool hasMouseTracking () const;
-	public slots:
+	public Q_SLOTS:
 		/*! Resizes the widget to size \p width by \p height pixels. See also width() and height(). */
 		virtual void resize(int width, int height);
 		/*! Sets the hasMouseTracking() value. */
@@ -452,7 +451,7 @@ protected:
 	/*! Returns \c true when buffers are automatically swapped (default). See details in the QGLWidget
 	documentation. */
 	bool autoBufferSwap() const;
-	protected slots:
+	protected Q_SLOTS:
 		/*! Sets the autoBufferSwap() value. */
 		void setAutoBufferSwap(bool on);
 		//@}
@@ -524,11 +523,11 @@ public:
 	int snapshotQuality() { return snapshotQuality_; };
 
 	// Qt 2.3 does not support double default value parameters in slots.
-	// Remove "slots" from the following line to compile with Qt 2.3
-	public slots:
+	// Remove "Q_SLOTS" from the following line to compile with Qt 2.3
+	public Q_SLOTS:
 	void saveSnapshot(bool automatic=true, bool overwrite=false);
 
-	public slots:
+	public Q_SLOTS:
 	void saveSnapshot(const QString& fileName, bool overwrite=false);
 	void setSnapshotFileName(const QString& name);
 
@@ -605,7 +604,7 @@ public:
 	float bufferTextureMaxU() const { return bufferTextureMaxU_; };
 	/*! Same as bufferTextureMaxU(), but for the v texture coordinate. */
 	float bufferTextureMaxV() const { return bufferTextureMaxV_; };
-	public slots:
+	public Q_SLOTS:
 		void copyBufferToTexture(GLint internalFormat, GLenum format=GL_NONE);
 		//@}
 
@@ -637,7 +636,7 @@ public:
 	animationIsStarted(), you should stopAnimation() first. */
 	int animationPeriod() const { return animationPeriod_; };
 
-	public slots:
+	public Q_SLOTS:
 		/*! Sets the animationPeriod(), in milliseconds. */
 		void setAnimationPeriod(int period) { animationPeriod_ = period; };
 		virtual void startAnimation();
@@ -649,17 +648,17 @@ public:
 		incremented in this method (frame-rate independent animation) or computed from actual time (for
 		instance using QTime::elapsed()) for real-time animations.
 
-		Note that KeyFrameInterpolator (which regularly updates a Frame) do not use this method but rather
-		rely on a QTimer signal-slot mechanism.
+                Note that KeyFrameInterpolator (which regularly updates a Frame) does not use this method
+                to animate a Frame, but rather rely on a QTimer signal-slot mechanism.
 
 		See the <a href="../examples/animation.html">animation example</a> for an illustration. */
-		virtual void animate() { emit animateNeeded(); };
+		virtual void animate() { Q_EMIT animateNeeded(); };
 		/*! Calls startAnimation() or stopAnimation(), depending on animationIsStarted(). */
 		void toggleAnimation() { if (animationIsStarted()) stopAnimation(); else startAnimation(); };
 		//@}
 
 public:
-signals:
+Q_SIGNALS:
 	/*! Signal emitted by the default init() method.
 
 	Connect this signal to the methods that need to be called to initialize your viewer or overload init(). */
@@ -748,7 +747,7 @@ public:
 	virtual QString shortcutBindingsString () const { return keyboardString(); }
 #endif
 
-	public slots:
+	public Q_SLOTS:
 		virtual void help();
 		virtual void aboutQGLViewer();
 
@@ -785,7 +784,7 @@ protected:
 
 	\note All the OpenGL specific initializations must be done in this method: the OpenGL context is
 	not yet available in your viewer constructor. */
-	virtual void init() { emit viewerInitialized(); };
+	virtual void init() { Q_EMIT viewerInitialized(); };
 
 	virtual void paintGL();
 	virtual void preDraw();
@@ -868,7 +867,7 @@ public:
 	the content of this buffer. See the \c glSelectBuffer() man page for details. */
 	GLuint* selectBuffer() { return selectBuffer_; };
 
-	public slots:
+	public Q_SLOTS:
 		virtual void select(const QMouseEvent* event);
 		virtual void select(const QPoint& point);
 
@@ -891,17 +890,15 @@ protected:
 	to be able to select. The default select() implementation relies on the \c GL_SELECT, and requires
 	that each selectable element is drawn within a \c glPushName() - \c glPopName() block. A typical
 	usage would be (see the <a href="../examples/select.html">select example</a>):
-	\code
-	void Viewer::drawWithNames()
-	{
-	for (int i=0; i<nbObjects; ++i)
-	{
-	glPushName(i);
-	object(i)->draw();
-	glPopName();
-	}
-	}
-	\endcode
+\code
+void Viewer::drawWithNames() {
+   for (int i=0; i<nbObjects; ++i) {
+      glPushName(i);
+      object(i)->draw();
+      glPopName();
+   }
+}
+\endcode
 
 	The resulting selected name is computed by endSelection(), which setSelectedName() to the integer
 	id pushed by this method (a value of -1 means no selection). Use selectedName() to update your
@@ -948,7 +945,7 @@ public:
 	QtKeyboardModifiers addKeyFrameKeyboardModifiers() const;
 	QtKeyboardModifiers playPathKeyboardModifiers() const;
 
-	public slots:
+	public Q_SLOTS:
 		void setShortcut(KeyboardAction action, int key);
 #ifndef DOXYGEN
 		void setKeyboardAccelerator(KeyboardAction action, int key);
@@ -1016,7 +1013,7 @@ public:
 	int wheelHandler(QtKeyboardModifiers modifiers) const;
 	int wheelButtonState(MouseHandler handler, MouseAction action, bool withConstraint=true) const;
 
-	public slots:
+	public Q_SLOTS:
 		void setMouseBinding(int state, MouseHandler handler, MouseAction action, bool withConstraint=true);
 #if QT_VERSION < 0x030000
 		// Two slots cannot have the same name or two default parameters with Qt 2.3.
@@ -1025,7 +1022,7 @@ public:
 	void setMouseBinding(int state, ClickAction action, bool doubleClick=false, QtMouseButtons buttonsBefore=Qt::NoButton);
 	void setMouseBindingDescription(int state, QString description, bool doubleClick=false, QtMouseButtons buttonsBefore=Qt::NoButton);
 #if QT_VERSION < 0x030000
-	public slots:
+	public Q_SLOTS:
 #endif
 		void setWheelBinding(QtKeyboardModifiers modifiers, MouseHandler handler, MouseAction action, bool withConstraint=true);
 		void setHandlerKeyboardModifiers(MouseHandler handler, QtKeyboardModifiers modifiers);
@@ -1046,7 +1043,7 @@ public:
 	QString stateFileName() const;
 	virtual QDomElement domElement(const QString& name, QDomDocument& document) const;
 
-	public slots:
+	public Q_SLOTS:
 		virtual void initFromDOMElement(const QDomElement& element);
 		virtual void saveStateToFile(); // cannot be const because of QMessageBox
 		virtual bool restoreStateFromFile();
@@ -1078,18 +1075,17 @@ private:
 	/*! @name QGLViewer pool */
 	//@{
 public:
-	/*! Returns a \c QList (see Qt documentation) that contains pointers to all the created
-	QGLViewers. Note that this list may contain \c NULL pointers if the associated viewer has been deleted.
+	/*! Returns a \c QList that contains pointers to all the created QGLViewers.
+        Note that this list may contain \c NULL pointers if the associated viewer has been deleted.
 
-	Can be useful to apply a method or to connect a signal to all the viewers.
+	Can be useful to apply a method or to connect a signal to all the viewers:
+        \code
+	foreach (QGLViewer* viewer, QGLViewer::QGLViewerPool())
+ 	  connect(myObject, SIGNAL(IHaveChangedSignal()), viewer, SLOT(updateGL()));
+	\endcode
 
 	\attention With Qt version 3, this method returns a \c QPtrList instead. Use a \c QPtrListIterator
-	to iterate on the list:
-	\code
-	QPtrListIterator<QGLViewer> it(QGLViewer::QGLViewerPool());
-	for (QGLViewer* viewer; (viewer = it.current()) != NULL; ++it)
-	connect(myObject, SIGNAL(mySignal), viewer, SLOT(updateGL()));
-	\endcode */
+	to iterate on the list instead.*/
 #if QT_VERSION >= 0x040000
 	static const QList<QGLViewer*>& QGLViewerPool() { return QGLViewer::QGLViewerPool_; };
 #else
@@ -1101,12 +1097,13 @@ public:
 	can be used to identify the different created QGLViewers (see stateFileName() for an application
 	example).
 
-	When a QGLViewer is deleted, the following QGLViewers' indexes are shifted down. Returns -1 if the
-	QGLViewer could not be found (which should not be possible). */
+	When a QGLViewer is deleted, the QGLViewers' indexes are preserved and NULL is set for that index.
+        When a QGLViewer is created, it is placed in the first available position in that list.
+        Returns -1 if the QGLViewer could not be found (which should not be possible). */
 #if QT_VERSION >= 0x040000
 	static int QGLViewerIndex(const QGLViewer* const viewer) { return QGLViewer::QGLViewerPool_.indexOf(const_cast<QGLViewer*>(viewer)); };
 #else
-	static int QGLViewerIndex(const QGLViewer* const viewer) { return QGLViewer::QGLViewerPool_.find(viewer); };
+	static int QGLViewerIndex(const QGLViewer* const viewer) { return QGLViewer::QGLViewerPool_.findRef(viewer); };
 #endif
 	//@}
 
@@ -1117,12 +1114,12 @@ public:
 	virtual void setVisualHintsMask(int mask, int delay = 2000);
 	virtual void drawVisualHints();
 
-	public slots:
+	public Q_SLOTS:
 		virtual void resetVisualHints();
 		//@}
 #endif
 
-		private slots:
+		private Q_SLOTS:
 			// Patch for a Qt bug with fullScreen on startup
 			void delayedFullScreen() { move(prevPos_); setFullScreen(); };
 			void hideMessage();
