@@ -1,8 +1,8 @@
 /****************************************************************************
 
- Copyright (C) 2002-2008 Gilles Debunne. All rights reserved.
+ Copyright (C) 2002-2011 Gilles Debunne. All rights reserved.
 
- This file is part of the QGLViewer library version 2.3.4.
+ This file is part of the QGLViewer library version 2.3.17.
 
  http://www.libqglviewer.com - contact@libqglviewer.com
 
@@ -34,10 +34,10 @@ Note that this rotation is not uniquely defined. The selected axis is usually or
 and \p to, minimizing the rotation angle. This method is robust and can handle small or almost identical vectors. */
 Quaternion::Quaternion(const Vec& from, const Vec& to)
 {
-  const float epsilon = 1E-10f;
+  const double epsilon = 1E-10f;
 
-  const float fromSqNorm = from.squaredNorm();
-  const float toSqNorm   = to.squaredNorm();
+  const double fromSqNorm = from.squaredNorm();
+  const double toSqNorm   = to.squaredNorm();
   // Identity Quaternion when one vector is null
   if ((fromSqNorm < epsilon) || (toSqNorm < epsilon))
     {
@@ -47,7 +47,7 @@ Quaternion::Quaternion(const Vec& from, const Vec& to)
   else
     {
       Vec axis = cross(from, to);
-      const float axisSqNorm = axis.squaredNorm();
+      const double axisSqNorm = axis.squaredNorm();
 
       // Aligned vectors, pick any axis, not aligned with from or to
       if (axisSqNorm < epsilon)
@@ -202,7 +202,7 @@ void Quaternion::getAxisAngle(Vec& axis, float& angle) const
 {
   angle = 2.0*acos(q[3]);
   axis = Vec(q[0], q[1], q[2]);
-  const float sinus = axis.norm();
+  const double sinus = axis.norm();
   if (sinus > 1E-8)
     axis /= sinus;
 
@@ -219,7 +219,7 @@ It is null for an identity Quaternion. See also angle() and getAxisAngle(). */
 Vec Quaternion::axis() const
 {
   Vec res = Vec(q[0], q[1], q[2]);
-  const float sinus = res.norm();
+  const double sinus = res.norm();
   if (sinus > 1E-8)
     res /= sinus;
   return (acos(q[3]) <= M_PI/2.0) ? res : -res;
@@ -231,9 +231,9 @@ Vec Quaternion::axis() const
  axis() direction.
 
  See also axis() and getAxisAngle(). */
-float Quaternion::angle() const
+double Quaternion::angle() const
 {
-  const float angle = 2.0 * acos(q[3]);
+  const double angle = 2.0 * acos(q[3]);
   return (angle <= M_PI) ? angle : 2.0*M_PI - angle;
 }
 
@@ -292,7 +292,7 @@ Quaternion::Quaternion(const QDomElement& element)
 #else
   for (unsigned int i=0; i<attribute.count(); ++i)
 #endif
-    q[i] = DomUtils::floatFromDom(element, attribute[i], ((i<3)?0.0f:1.0f));
+    q[i] = DomUtils::doubleFromDom(element, attribute[i], ((i<3)?0.0f:1.0f));
 }
 
 /*! Returns the Quaternion associated 4x4 OpenGL rotation matrix.
@@ -436,9 +436,9 @@ void Quaternion::getInverseRotationMatrix(float m[3][3]) const
  negate()). */
 Quaternion Quaternion::slerp(const Quaternion& a, const Quaternion& b, float t, bool allowFlip)
 {
-  float cosAngle = Quaternion::dot(a, b);
+  double cosAngle = Quaternion::dot(a, b);
 
-  float c1, c2;
+  double c1, c2;
   // Linear interpolation for close orientations
   if ((1.0 - fabs(cosAngle)) < 0.01)
     {
@@ -448,8 +448,8 @@ Quaternion Quaternion::slerp(const Quaternion& a, const Quaternion& b, float t,
   else
     {
       // Spherical interpolation
-      float angle    = acos(fabs(cosAngle));
-      float sinAngle = sin(angle);
+      double angle    = acos(fabs(cosAngle));
+      double sinAngle = sin(angle);
       c1 = sin(angle * (1.0 - t)) / sinAngle;
       c2 = sin(angle * t) / sinAngle;
     }
@@ -478,13 +478,13 @@ Quaternion Quaternion::squad(const Quaternion& a, const Quaternion& tgA, const Q
 /*! Returns the logarithm of the Quaternion. See also exp(). */
 Quaternion Quaternion::log()
 {
-  float len = sqrt(q[0]*q[0] + q[1]*q[1] + q[2]*q[2]);
+  double len = sqrt(q[0]*q[0] + q[1]*q[1] + q[2]*q[2]);
 
   if (len < 1E-6)
     return Quaternion(q[0], q[1], q[2], 0.0);
   else
     {
-      float coef = acos(q[3]) / len;
+      double coef = acos(q[3]) / len;
       return Quaternion(q[0]*coef, q[1]*coef, q[2]*coef, 0.0);
     }
 }
@@ -492,13 +492,13 @@ Quaternion Quaternion::log()
 /*! Returns the exponential of the Quaternion. See also log(). */
 Quaternion Quaternion::exp()
 {
-  float theta = sqrt(q[0]*q[0] + q[1]*q[1] + q[2]*q[2]);
+  double theta = sqrt(q[0]*q[0] + q[1]*q[1] + q[2]*q[2]);
 
   if (theta < 1E-6)
     return Quaternion(q[0], q[1], q[2], cos(theta));
   else
     {
-      float coef = sin(theta) / theta;
+      double coef = sin(theta) / theta;
       return Quaternion(q[0]*coef, q[1]*coef, q[2]*coef, cos(theta));
     }
 }
@@ -547,10 +547,10 @@ Quaternion Quaternion::randomQuaternion()
 {
   // The rand() function is not very portable and may not be available on your system.
   // Add the appropriate include or replace by an other random function in case of problem.
-  double seed = rand()/(float)RAND_MAX;
+  double seed = rand()/(double)RAND_MAX;
   double r1 = sqrt(1.0 - seed);
   double r2 = sqrt(seed);
-  double t1 = 2.0 * M_PI * (rand()/(float)RAND_MAX);
-  double t2 = 2.0 * M_PI * (rand()/(float)RAND_MAX);
+  double t1 = 2.0 * M_PI * (rand()/(double)RAND_MAX);
+  double t2 = 2.0 * M_PI * (rand()/(double)RAND_MAX);
   return Quaternion(sin(t1)*r1, cos(t1)*r1, sin(t2)*r2, cos(t2)*r2);
 }
